# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - Check the speed of page editing

    Check to see how quickly the user wanted to edit a page after previous
    actions. Prevent them from saving a page if they were too fast.

    @copyright: 2016 Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL, see COPYING for details.
"""

from MoinMoin.events import PagePreSaveEvent, Abort
from MoinMoin.logfile.eventlog import EventLog
from MoinMoin import wikiutil
import time

# The default editing delay where no configuration setting has been provided.
# Value expressed in seconds.

editing_delay_default = "10"

# Event handling.

def handle_presave(event):
    request = event.request
    _ = request.getText

    # Obtain user details. If no user is identified, nothing can be done.

    username = request.user.name
    if not username:
        return None

    # Obtain the required delay between viewing and editing. This is expressed
    # in seconds.

    delay = float(getattr(request.cfg, "editing_delay", editing_delay_default))

    # Obtain the current time.

    time_now = time.time()

    # Check in the log for the last action by the user. This requires the
    # recording of the username in the log.

    log = EventLog(request)
    log.set_filter(["VIEWPAGE", "SAVEPAGE", "SAVENEW"])

    pagename = event.page_editor.page_name
    last_viewed = None
    last_saved = None

    for event in log.reverse():
        event_usecs, event_type, event_attr = event[:3]

        # Convert to seconds and determine if the start of the delay period has
        # been reached.

        event_secs = wikiutil.version2timestamp(event_usecs)
        if time_now - event_secs >= delay:
            break

        event_page = event_attr.get("pagename")
        event_user = event_attr.get("username")

        # Stop when encountering the latest event entry for this page and user.

        if event_page == pagename and event_user == username:
            if event_type == "VIEWPAGE":
                last_viewed = event_secs
            else:
                last_saved = event_secs
            break

    # Any attempt to save occurring too soon after VIEWPAGE is possibly an
    # automated edit.

    if last_viewed:
        return Abort(_("Page not changed: the page was saved too quickly after viewing."))

    # Any attempt to save without any previous VIEWPAGE, but after a SAVENEW or
    # SAVEPAGE may also be an automated edit.

    elif last_saved:
        return Abort(_("Page not changed: the page was saved again too soon or saved without viewing the page."))

    return None

def handle(event):
    if isinstance(event, PagePreSaveEvent):
        return handle_presave(event)

# vim: tabstop=4 expandtab shiftwidth=4
